#======================================================================================#
# This script is an example showing how to do rigid grid modelling of spectra          #
# Cassis scripting documentation : http://cassis.irap.omp.eu/docs/script/README.html   #
#======================================================================================#

import ScriptEnvironment
from Range import Range
from Component import Component
from LineAnalysisScripting import UserInputs
from eu.omp.irap.cassis.properties import Software
from cassisStats import writeStats
from java.io import File
from Plot import Plot

import os
import codecs
import unicodedata


#======================================================================================#
# Inputs :                                                                             #
#======================================================================================#
sourceName = 'test_source'
myModel    = "LTE"

#inputfile_loc =  '<link-to-input-txt-folder>' # End with / so that it is indeed a folder
#inputfile_name = '<name-of-input-txt>' # Name of input .txt file
#spectra_loc = '<link-to-spectra-folder>' # End with / so that it is indeed a folder
#spectra_name = '<name-of-spectrum-file>' # Name of spectrum .fus file
#file_out_loc = '<link-to-output-folder>' # End with / so that it is indeed a folder
#telescope_loc = '<link-to-telescope-folder>' # End with / so that it is indeed a folder
#telescope_name = '<name-of-telescope>' # Name of telescope file. e.g. alma_400, NO extenstion needed

inputfile_loc   =  '<link-to-input-txt-folder>' # End with / so that it is indeed a folder
inputfile_name  = 'grid_LTE_input.txt' # Name of input .txt file
spectra_loc     = '<link-to-spectra-folder>' # End with / so that it is indeed a folder
spectra_name    = 'Synthetic_LTE_1_broadfreq.fus' # Name of spectrum .fus file
file_out_loc    = '<link-to-output-folder>' # End with / so that it is indeed a folder
telescope_loc   = '<link-to-telescope-folder>' # End with / so that it is indeed a folder
telescope_name  = 'alma_400m' # Name of telescope file. e.g. alma_400m, NO extenstion needed

rms = 0.2 # rms in Kelvin
cal_error = 0.2 # calibration error in %/100
spectral_range = [210,230] # Frequency range of the data in GHz
tuning_band = 60 # bandwidth over which single spectral lines are plotted
aij_min = 0 # minimum Aij (NOTE: be careful: if you change this, you also change the line numbering)


#======================================================================================#
# Fitting procedure :                                                                  #
#======================================================================================#

# Load in input .txt file and start loop over all lines
print(inputfile_loc + inputfile_name)
data_all = open(inputfile_loc + inputfile_name, 'r')

for data in data_all:
    #======================================================================================#
    # Reading in our files and converting to CASSIS format :                               #
    #======================================================================================#
    # Exclude lines which start with #
    if data[0] == "#" or len(data) < 10:
        continue
    
    # Read in the lines (be very carefull in changing the input .txt file to keep the right format)
    data_split = data.split()
    print(data_split)
    try:
        speciesName, speciesCatalogue, speciesTag, vel_range, Eup_range, N_range, T_range, FWHM_range, size_range, vlsr_range, lines = data_split
        print('Now running for ' + speciesName)
    except:
        print('Error in loading data line', 'Continuing to next line')
        continue

    # Convert speciestag to CASSIS format
    speciesTag = int(speciesTag)
    speciesCatalogue = speciesCatalogue.replace('_',' ')
    
    # Convert range of Eup to be included to CASSIS format
    Eup_range = Eup_range.split('_')
    Eup_range = [float(j) for j in Eup_range]
    
    # Setup grid of column densities and convert to CASSIS format
    N_niter = float(N_range.split(',')[-1])
    N_range = N_range.split(',')[0].split('_')
    N_range = [float(j) for j in N_range]
    if N_range[0] < 100:
        if N_niter < 10:
            N_niter = round((N_range[1]-N_range[0])/N_niter) + 1
        N_range[0] = 10**N_range[0]
        N_range[1] = 10**N_range[1]
    N_niter = int(N_niter)

    # Setup grid of temperatures and convert to CASSIS format
    if ',' in T_range:
        T_niter = float(T_range.split(',')[-1])
        T_range = T_range.split(',')[0].split('_')
        T_range = [float(j) for j in T_range]
        if T_niter <= 100:
            T_niter = round((T_range[1]-T_range[0])/T_niter) + 1
        T_niter = int(T_niter)
    else:
        T_range = [float(T_range), float(T_range)]
        T_niter = 1   
    
    # Setup grid of FWHM and convert to CASSIS format
    if ',' in FWHM_range:
        FWHM_niter = float(FWHM_range.split(',')[-1])
        FWHM_range = FWHM_range.split(',')[0].split('_')
        FWHM_range = [float(j) for j in FWHM_range]
        if FWHM_niter <= 10.:
            FWHM_niter = round((FWHM_range[1]-FWHM_range[0])/FWHM_niter) + 1
        FWHM_niter = int(FWHM_niter)
    else:
        FWHM_range = [float(FWHM_range),float(FWHM_range)]
        FWHM_niter = 1
    
    # Setup grid of vlsr and convert to CASSIS format
    if ',' in vlsr_range:
        vlsr_niter = float(vlsr_range.split(',')[-1])
        vlsr_range = vlsr_range.split(',')[0].split('_')
        vlsr_range = [float(j) for j in vlsr_range]
        if vlsr_niter <= 10.:
            vlsr_niter = round((vlsr_range[1]-vlsr_range[0])/vlsr_niter) + 1
        vlsr_niter = int(vlsr_niter)
    else:
        vlsr_range = [float(vlsr_range),float(vlsr_range)]
        vlsr_niter = 1
    
    # Convert velocity over which the spectral lines will be fitted to CASSIS format
    vel_range = vel_range.split('_')
    vel_range = [float(j) for j in vel_range]
    Range.unit          = "km/s"    # Possible units are GHz and km/s, MHz, cm-1 and micrometer
    v1                  = Range(vlsr_range[0] + vel_range[0], vlsr_range[1] + vel_range[1])
    print(v1)
    # Setup grid of source sizes and convert to CASSIS format
    if ',' in size_range:
        size_niter = float(size_range.split(',')[-1])
        size_range = size_range.split(',')[0].split('_')
        size_range = [float(j) for j in size_range]
        if size_niter <= 10.:
            size_niter = round((size_range[1]-size_range[0])/size_niter) + 1
        size_niter = int(size_niter)
    else:
        size_range = [float(size_range),float(size_range)]
        size_niter = 1
    
    # Read in the line numbers to be used (a 0 means continue to next)
    lines.replace('_', '-')
    if '\n' in lines:
        lines = lines[:-1]
    if '0' in lines and len(lines) == 2:
        continue
    
    # Set the lines to fit, rms, and absolute flux calibration error in CASSIS format
    if not lines == "all":
        lines_sel = {str(lines):[v1]}
        lines_rms = {str(lines):[rms]}
        lines_cal = {str(lines):[cal_error]}
    else:
        lines_sel = {str('*-*'):[v1]}
        lines_rms = {str('*-*'):[rms]}
        lines_cal = {str('*-*'):[cal_error]}
        
    # Set the correct file names and folders for input/output
    myName        = sourceName+"_"+speciesName+"_"+myModel
    myDirInput    = spectra_loc
    myDirOutput   = file_out_loc + speciesName + '/'
    inputFile     =  myDirInput + spectra_name
    telescopeFile = telescope_loc + telescope_name
    outputFile    = myDirOutput+myName+".dat"
    
    # Create output folder if it does not exist yet
    if not os.path.isdir(myDirOutput):
        os.makedirs(myDirOutput)
        
    #======================================================================================#
    # Generating CASSIS model infput :                                                     #
    #======================================================================================#
    
    # Function of CASSIS that reads in our user input in CASSIS format
    userInputs          = UserInputs(
    inputFile           = inputFile,
    telescope           = telescopeFile,
    tuningRange         = spectral_range,
    tuningBand          = tuning_band,
    aijMin              = aij_min,
    eup                 = Eup_range,
    template            = speciesCatalogue,
    moltags             = [speciesTag],
    tmb2ta              = False,
    isoUnique           = False,
    outputFile          = outputFile,
    plotTitle           = myName,
    warning             = False,
    selectedLines       = lines_sel,
    rmsLines            = lines_rms,
    calLines            = lines_cal
    )
    
    #======================================================================================#
    # Parameters to be defined for all components :                                        #
    #======================================================================================#
    
    # Type of models :
    # LTE  : nmol, tex, fwhm, size, vlsr and iso if there are different moltags
    # RADEX: nmol, collisionFile, n_collisioners, tkin, fwhm, size, vlsr and iso
    
    comp_1              = Component(
    # Needed for any model
    nmol                = {'min':N_range[0], 'max':N_range[1], 'nstep':N_niter,  'log_mode':True},
    temp                = {'min':T_range[0], 'max':T_range[1],   'nstep':T_niter,  'log_mode':False},
    fwhm                = {'min':FWHM_range[0],    'max':FWHM_range[1],    'nstep':FWHM_niter,  'log_mode':False},
    size                = {'min':size_range[0],     'max':size_range[1],     'nstep':size_niter,  'log_mode':False},
    vlsr                = {'min':vlsr_range[0],    'max':vlsr_range[1],    'nstep':vlsr_niter,  'log_mode':False},
    iso                 = {'min':1,     'max':1,     'nstep':1,  'log_mode':False},
    #continuum           = "continuum-0",
    interacting         = True,
    model               = myModel,
    # only needed for RADEX 
    collisioner         = ["p_H2","o_H2"], 
    n_p_H2              = {'min':2.0e8,  'max':2.0e8,  'nstep':1,   'log_mode':False},
    n_o_H2              = {'min':7.0e6,  'max':7.0e6,  'nstep':1,   'log_mode':False},
    collisionFile       = ["o-h2co_h2.dat","p-h2co_h2.dat"],
    geometry            = "sphere"
    )
    
    #======================================================================================#
    # Execute fitting procedure :                                                          #
    #======================================================================================#
    
    # Compute the chi2 min using the regular grid
    userInputs.computeChi2MinUsingRG(comp_1)  
 
    
    #======================================================================================#
    # Analysis of the results :                                                            #
    #======================================================================================#
    
    # Plot the best model and save the corresponding spectra and config files
    lineModel           = userInputs.plotBestModel(moltag = speciesTag, overSampling=3, tuningBand=tuning_band, telescope=telescopeFile)
    lineModel.saveConfig(File(myDirOutput+myName+".lam"))
    bestPhysicalModels  = userInputs.getBestPhysicalModels()
    userInputs.saveBestPhysicalModels(myDirOutput+myName+"_bestModel.lis")
    
    # Compute the chi2 contour plot
    try:
        userInputs.plotChi2Contour()
    except:
        print('Chi2 plotting failed, possibly because only one free parameter was used.')



