#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Fri Feb  4 15:17:38 2022

@author: vgelder
"""

import os
import numpy as np
import matplotlib.pyplot as plt
import argparse

#======================================================================================#
# Inputs (only need to change things here) :                                           #
# Run script as: python create_contour_plots.py <name_of_dat_file>.dat#
#======================================================================================#
def input_parameters():
    output_loc = os.getcwd() + '/contour_plots/'
    x_axis = 'Tex'
    return [x_axis,output_loc]

#======================================================================================#
# Main function:                                                                       #
#======================================================================================#
def main():
    man_input = input_parameters()
    chi2_files = parseArguments()
    for chi2_file in chi2_files.file:
        chi2_file_data, chi2_file_loc = load_chi2_file(chi2_file)
        
        master_dic, param_arrays, bestval_array, dof = crete_master_dic(chi2_file_data)
        contour_plots = convert_to_contourplots(man_input, master_dic, param_arrays, bestval_array)
        
        plot_contourplots(man_input, contour_plots,  param_arrays, bestval_array, dof)
        

#======================================================================================#
# Function that creates the contour plots:                                             #
#======================================================================================#        
def plot_contourplots(man_input, contour_plots, param_arrays, bestval_array, dof):
    x_axis, output_loc = man_input
    tsize = 15
    
    coeff_all = chi2_coef(dof)
    sigma_contour_to_plot = [1,2,3]
    
    link_i_to_axis = {0: 'N', 1:'Tex', 2:'FWHM', 3:'size', 4:'vlsr'}
    axes_labels_dic = {'N': r'$\log_{10}$ $N$ (cm$^{-2}$)', 'Tex': '$T_\mathrm{ex}$ (K)', 'FWHM': 'FWHM (kms/s)', 'size': '$\theta_\mathrm{source}$ (arcsec)', 'vlsr': '$V_\mathrm{lsr}$ (km/s)'}
    link_axis_to_i = dict((v, k) for k, v in link_i_to_axis.items())
    i_xaxis = link_axis_to_i[x_axis]
    for i,contour_plot in enumerate(contour_plots):
        y_axis = link_i_to_axis[i]
        if len(contour_plot) < 5 or len(contour_plot[0]) < 5:
            continue
        if x_axis == y_axis:
            continue
        best_x, best_y = bestval_array[i_xaxis], bestval_array[i]
        bext_x_idx = np.where(param_arrays[i_xaxis] == best_x)[0][0]
        bext_y_idx = np.where(param_arrays[i] == best_y)[0][0]

        contour_levels = []
        for sigma_contour in sigma_contour_to_plot:
            coef = coeff_all[sigma_contour-1]
            sigma_value = np.min(contour_plot + coef)
            contour_levels.append(sigma_value)

        axes_labels = axes_labels_dic[x_axis], axes_labels_dic[y_axis]
        plt.figure(figsize=(8,6))
        ax = plt.subplot(1,1,1)
        ax.contour(contour_plot, levels=contour_levels, colors='k')
        ax.scatter(bext_x_idx, bext_y_idx, marker='x', s=20, c='k')
        set_figure_axes(ax, axes_labels , i_xaxis, i, contour_plot, param_arrays, tsize)
        print(output_loc)
        os.makedirs(output_loc, exist_ok=True)
        plt.savefig(output_loc + 'Contour_' + x_axis + '_' + y_axis + '.pdf', bbox_inches='tight')
        
#======================================================================================#
# Function setting the axes params:                                                    #
#======================================================================================#
def set_figure_axes(ax, axes_labels, i_xaxis, i_yaxis, contour_plot, param_arrays, tsize):
    x_axis_values = param_arrays[i_xaxis]
    y_axis_values = param_arrays[i_yaxis]
    
    nr_x_ticks = 5
    nr_y_ticks = 5
    
    x_tick_values = np.linspace(0., np.shape(contour_plot)[1]-1, nr_x_ticks)
    x_tick_labels = np.array(['%.1f' % xval for xval in np.linspace(np.min(x_axis_values), np.max(x_axis_values), nr_x_ticks)])
    
    y_tick_values = np.linspace(0., np.shape(contour_plot)[0]-1, nr_y_ticks)
    y_tick_labels = np.array(['%.1f' % yval for yval in np.linspace(np.min(y_axis_values), np.max(y_axis_values), nr_y_ticks)])
    
    ax.tick_params(labelsize=tsize)
    ax.set_xticks(x_tick_values)
    ax.set_yticks(y_tick_values)
    ax.tick_params(axis='both', which='major', labelsize=tsize)
    ax.tick_params(axis='both', which='minor', labelsize=tsize)
    ax.set_xticklabels(x_tick_labels, fontsize=tsize)
    ax.set_yticklabels(y_tick_labels, fontsize=tsize)
    ax.set_xlabel(axes_labels[0], fontsize=tsize)
    ax.set_ylabel(axes_labels[1], fontsize=tsize)


#======================================================================================#
# Function converting data to 2D contour format:                                       #
#======================================================================================#
def convert_to_contourplots(man_input, master_dic, param_arrays, bestval_array, undef_val=-1):
    x_axis, _ = man_input
    
    N_arr, Tex_arr, FWHM_arr, size_arr, vlsr_arr = param_arrays
    
    N_x, Tex_x, FWHM_x, size_x, vlsr_x = [False]*5
    if x_axis == 'N':
        N_x = True
        x_data = N_arr
    if x_axis == 'Tex':
        Tex_x = True
        x_data = Tex_arr
    if x_axis == 'FWHM':
        FWHM_x = True
        x_data = FWHM_arr
    if x_axis == 'size':
        size_x = True
        x_data = size_arr
    if x_axis == 'vlsr':
        vlsr_x = True
        x_data = vlsr_arr

    x_data = np.sort(x_data)
    N_2D, Tex_2D, FWHM_2D, size_2D, vlsr_2D = [],[],[],[],[]
    N_best, Tex_best, FWHM_best, size_best, vlsr_best = bestval_array
    for x_val in x_data:
        if N_x:
            N_best = x_val
        if Tex_x:
            Tex_best = x_val
        if FWHM_x:
            FWHM_best = x_val
        if size_x:
            size_best = x_val
        if vlsr_x:
            vlsr_best = x_val
        
        N_temp, Tex_temp, FWHM_temp, size_temp, vlsr_temp = [],[],[],[],[]
        for N in N_arr:
            try:
                N_temp.append(master_dic[N][Tex_best][FWHM_best][size_best][vlsr_best])
            except KeyError:
                N_temp.append({})
        N_2D.append(N_temp)
        for Tex in Tex_arr:
            try:
                Tex_temp.append(master_dic[N_best][Tex][FWHM_best][size_best][vlsr_best])
            except KeyError:
                Tex_temp.append({})
        Tex_2D.append(Tex_temp)
        for FWHM in FWHM_arr:
            try:
                FWHM_temp.append(master_dic[N_best][Tex_best][FWHM][size_best][vlsr_best])
            except KeyError:
                FWHM_temp.append({})
        FWHM_2D.append(FWHM_temp)
        for size in size_arr:
            try:
                size_temp.append(master_dic[N_best][Tex_best][FWHM_best][size][vlsr_best])
            except KeyError:
                size_temp.append({})
        size_2D.append(size_temp)
        for vlsr in vlsr_arr:
            try:
                vlsr_temp.append(master_dic[N_best][Tex_best][FWHM_best][size_best][vlsr])
            except KeyError:
                vlsr_temp.append({})
        vlsr_2D.append(vlsr_temp)

    N_2D = np.transpose(N_2D)
    N_2D[np.where(N_2D == {})] = undef_val
    Tex_2D = np.transpose(Tex_2D)
    Tex_2D[np.where(Tex_2D == {})] = undef_val
    FWHM_2D = np.transpose(FWHM_2D)
    FWHM_2D[np.where(FWHM_2D == {})] = undef_val
    size_2D = np.transpose(size_2D)
    size_2D[np.where(size_2D == {})] = undef_val
    vlsr_2D = np.transpose(vlsr_2D)
    vlsr_2D[np.where(vlsr_2D == {})] = undef_val
    
    return N_2D, Tex_2D, FWHM_2D, size_2D, vlsr_2D
        
 
#======================================================================================#
# Function creating a master dic from data:                                            #
#======================================================================================#
def crete_master_dic(chi2_file_data):
    dof, nbPoints, N_x_full, T_ex_full, fwhm_full, size_full, vlsr_full, chi2_full  = chi2_file_data
    chi2_best = np.min(chi2_full)
    master_dic = {}
    N_x_array, T_ex_array, fwhm_array, size_array, vlsr_array = [], [], [], [], []
    for i in range(len(chi2_full)):
        N_x_i = N_x_full[i]
        T_ex_i = T_ex_full[i]
        fwhm_i = fwhm_full[i]
        size_i = size_full[i]
        vlsr_i = vlsr_full[i]
        
        if not N_x_i in N_x_array:
            N_x_array.append(N_x_i)
        if not T_ex_i in T_ex_array:
            T_ex_array.append(T_ex_i)
        if not fwhm_i in fwhm_array:
            fwhm_array.append(fwhm_i)
        if not size_i in size_array:
            size_array.append(size_i)
        if not vlsr_i in vlsr_array:
            vlsr_array.append(vlsr_i)
        
        if not N_x_i in master_dic.keys():
            master_dic[N_x_i] = {}
        if not T_ex_i in master_dic[N_x_i].keys():
            master_dic[N_x_i][T_ex_i] = {}
        if not fwhm_i in master_dic[N_x_i][T_ex_i].keys():
            master_dic[N_x_i][T_ex_i][fwhm_i] = {}
        if not size_i in master_dic[N_x_i][T_ex_i][fwhm_i].keys():
            master_dic[N_x_i][T_ex_i][fwhm_i][size_i] = {}
        zData = chi2_full[i] / (chi2_best/(nbPoints-dof))
        master_dic[N_x_i][T_ex_i][fwhm_i][size_i][vlsr_i] = zData
        
        if chi2_full[i] == chi2_best:
            bestval_array = np.array([N_x_i, T_ex_i, fwhm_i, size_i, vlsr_i])
    
    N_x_array, T_ex_array, fwhm_array, size_array, vlsr_array = np.array(N_x_array), np.array(T_ex_array), np.array(fwhm_array), np.array(size_array), np.array(vlsr_array)
    param_arrays = N_x_array, T_ex_array, fwhm_array, size_array, vlsr_array
    return master_dic, param_arrays, bestval_array, dof
        
    
#======================================================================================#
# Function loading chi2 data                   :                                       #
#======================================================================================#
def load_chi2_file(chi2_file):
    chi2_file_name = chi2_file.name
    chi2_file_loc_split = chi2_file_name.split('/')[:-1]
    chi2_file_loc = ''
    for line in chi2_file_loc_split:
        chi2_file_loc = chi2_file_loc + line + '/'
        
    header = np.loadtxt(chi2_file, max_rows=1, dtype=str)
    dof = int(header[2].split('=')[1])
    nbPoints = int(header[3].split('=')[1])
    data = np.loadtxt(chi2_file, skiprows=1).T
    _, N, T_ex, FWHM, source_size, vlsr, _, chi2  = data
    
    chi2_file_data = dof, nbPoints, N, T_ex, FWHM, source_size, vlsr, chi2
    return chi2_file_data, chi2_file_loc

#======================================================================================#
# Function computing chi2 coefficient for contour plots:                               #
#======================================================================================#
def chi2_coef(dof):
    chi_coef_dic = {1.:np.array([1.00000, 4.00000, 9.00000]), \
                    2.:np.array([2.29000, 6.17000, 11.8100]), \
                    3.:np.array([3.53000, 8.02000, 14.1400]), \
                    4.:np.array([4.72000, 9.71000, 16.2300]), \
                    5.:np.array([5.89000, 11.3100, 18.1800]), \
                    6.:np.array([7.04000, 12.8400, 20.0300]), \
                    7.:np.array([8.17000, 14.3300, 21.7600]), \
                    8.:np.array([9.30000, 15.7700, 23.3800]), \
                    9.:np.array([10.4200, 17.1800, 24.8500]), \
                   10.:np.array([11.5300, 18.5500, 26.1400])}
    try:
        coef = chi_coef_dic[dof]
    except KeyError:
        print('')
        print('chi_2 coeff dictionary does not have ' + str(dof) + 'entry. Please add this (in CASSIS_funcs.chi2_coef)')
        print('')
        raise
    return coef

#======================================================================================#
# Function reading supplied file:                                                      #
#======================================================================================#
def parseArguments():
    parser = argparse.ArgumentParser(description="Plots a 1D spectrum.")
    parser.add_argument('file', type=argparse.FileType('rb'), nargs='*')
#    parser.add_argument("-o", "--open", type=str, dest="filename", \
#                    default=False, help="Fitsfile to open")

    fileName = parser.parse_args()

    return fileName 
    
#======================================================================================#
# Run main function:                                                                   #
#======================================================================================#
if __name__ == '__main__':
    main()